@@ -22,10 +22,12 @@ import (
 	"strings"
 	"time"
 
+	monitoring "github.com/coreos/prometheus-operator/pkg/apis/monitoring"
 	monitoringv1 "github.com/coreos/prometheus-operator/pkg/apis/monitoring/v1"
 	monitoringclient "github.com/coreos/prometheus-operator/pkg/client/versioned"
 	"github.com/coreos/prometheus-operator/pkg/k8sutil"
 	"github.com/coreos/prometheus-operator/pkg/listwatch"
+	"github.com/coreos/prometheus-operator/pkg/operator"
 
 	"github.com/go-kit/kit/log"
 	"github.com/go-kit/kit/log/level"
@@ -74,14 +76,11 @@ type Operator struct {
 
 	queue workqueue.RateLimitingInterface
 
-	reconcileErrorsCounter *prometheus.CounterVec
+	metrics *operator.Metrics
 
-	// triggerByCounter is a set of counters keeping track of the amount
-	// of times Prometheus Operator was triggered to reconcile its created
-	// objects. It is split in the dimensions of Kubernetes objects and
-	// corresponding actions (add, delete, update).
-	triggerByCounter        *prometheus.CounterVec
 	nodeAddressLookupErrors prometheus.Counter
+	nodeEndpointSyncs       prometheus.Counter
+	nodeEndpointSyncErrors  prometheus.Counter
 
 	host                   string
 	kubeletObjectName      string
@@ -144,10 +143,8 @@ type Config struct {
 	AlertmanagerDefaultBaseImage  string
 	PrometheusDefaultBaseImage    string
 	ThanosDefaultBaseImage        string
-	Namespaces                    []string
-	DeniedNamespaces              []string
+	Namespaces                    Namespaces
 	Labels                        Labels
-	CrdGroup                      string
 	CrdKinds                      monitoringv1.CrdKinds
 	EnableValidation              bool
 	LocalHost                     string
@@ -158,13 +155,30 @@ type Config struct {
 	AlertManagerSelector          string
 }
 
+type Namespaces struct {
+	// allow list/deny list for common custom resources
+	AllowList, DenyList []string
+	// allow list for prometheus/alertmanager custom resources
+	PrometheusAllowList, AlertmanagerAllowList []string
+}
+
+// BasicAuthCredentials represents a username password pair to be used with
+// basic http authentication, see https://tools.ietf.org/html/rfc7617.
 type BasicAuthCredentials struct {
 	username string
 	password string
 }
 
+// BearerToken represents a bearer token, see
+// https://tools.ietf.org/html/rfc6750.
+type BearerToken string
+
+// TLSAsset represents any TLS related opaque string, e.g. CA files, client
+// certificates.
+type TLSAsset string
+
 // New creates a new controller.
-func New(conf Config, logger log.Logger) (*Operator, error) {
+func New(conf Config, logger log.Logger, r prometheus.Registerer) (*Operator, error) {
 	cfg, err := k8sutil.NewClusterConfig(conf.Host, conf.TLSInsecure, &conf.TLSConfig)
 	if err != nil {
 		return nil, errors.Wrap(err, "instantiating cluster config failed")
@@ -218,88 +232,117 @@ func New(conf Config, logger log.Logger) (*Operator, error) {
 		kubeletObjectNamespace: kubeletObjectNamespace,
 		kubeletSyncEnabled:     kubeletSyncEnabled,
 		config:                 conf,
-		configGenerator:        NewConfigGenerator(logger),
+		configGenerator:        newConfigGenerator(logger),
+		metrics:                operator.NewMetrics("prometheus", r),
+		nodeAddressLookupErrors: prometheus.NewCounter(prometheus.CounterOpts{
+			Name: "prometheus_operator_node_address_lookup_errors_total",
+			Help: "Number of times a node IP address could not be determined",
+		}),
+		nodeEndpointSyncs: prometheus.NewCounter(prometheus.CounterOpts{
+			Name: "prometheus_operator_node_syncs_total",
+			Help: "Number of node endpoints synchronisations",
+		}),
+		nodeEndpointSyncErrors: prometheus.NewCounter(prometheus.CounterOpts{
+			Name: "prometheus_operator_node_syncs_failed_total",
+			Help: "Number of node endpoints synchronisation failures",
+		}),
 	}
+	c.metrics.MustRegister(c.nodeAddressLookupErrors, c.nodeEndpointSyncs, c.nodeEndpointSyncErrors)
 
 	c.promInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
-			return &cache.ListWatch{
-				ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
-					options.LabelSelector = c.config.PromSelector
-					return mclient.MonitoringV1().Prometheuses(namespace).List(options)
-				},
-				WatchFunc: func(options metav1.ListOptions) (watch.Interface, error) {
-					options.LabelSelector = c.config.PromSelector
-					return mclient.MonitoringV1().Prometheuses(namespace).Watch(options)
-				},
-			}
-		}),
+		c.metrics.NewInstrumentedListerWatcher(
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces.PrometheusAllowList, c.config.Namespaces.DenyList, func(namespace string) cache.ListerWatcher {
+				return &cache.ListWatch{
+					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
+						options.LabelSelector = c.config.PromSelector
+						return mclient.MonitoringV1().Prometheuses(namespace).List(options)
+					},
+					WatchFunc: func(options metav1.ListOptions) (watch.Interface, error) {
+						options.LabelSelector = c.config.PromSelector
+						return mclient.MonitoringV1().Prometheuses(namespace).Watch(options)
+					},
+				}
+			}),
+		),
 		&monitoringv1.Prometheus{}, resyncPeriod, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc},
 	)
+	c.metrics.MustRegister(NewPrometheusCollector(c.promInf.GetStore()))
 
 	c.smonInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
-			return &cache.ListWatch{
-				ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
-					return mclient.MonitoringV1().ServiceMonitors(namespace).List(options)
-				},
-				WatchFunc: mclient.MonitoringV1().ServiceMonitors(namespace).Watch,
-			}
-		}),
+		c.metrics.NewInstrumentedListerWatcher(
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces.AllowList, c.config.Namespaces.DenyList, func(namespace string) cache.ListerWatcher {
+				return &cache.ListWatch{
+					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
+						return mclient.MonitoringV1().ServiceMonitors(namespace).List(options)
+					},
+					WatchFunc: mclient.MonitoringV1().ServiceMonitors(namespace).Watch,
+				}
+			}),
+		),
 		&monitoringv1.ServiceMonitor{}, resyncPeriod, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc},
 	)
 
 	c.pmonInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
-			return &cache.ListWatch{
-				ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
-					return mclient.MonitoringV1().PodMonitors(namespace).List(options)
-				},
-				WatchFunc: mclient.MonitoringV1().PodMonitors(namespace).Watch,
-			}
-		}),
+		c.metrics.NewInstrumentedListerWatcher(
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces.AllowList, c.config.Namespaces.DenyList, func(namespace string) cache.ListerWatcher {
+				return &cache.ListWatch{
+					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
+						return mclient.MonitoringV1().PodMonitors(namespace).List(options)
+					},
+					WatchFunc: mclient.MonitoringV1().PodMonitors(namespace).Watch,
+				}
+			}),
+		),
 		&monitoringv1.PodMonitor{}, resyncPeriod, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc},
 	)
 
 	c.ruleInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
-			return &cache.ListWatch{
-				ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
-					return mclient.MonitoringV1().PrometheusRules(namespace).List(options)
-				},
-				WatchFunc: mclient.MonitoringV1().PrometheusRules(namespace).Watch,
-			}
-		}),
+		c.metrics.NewInstrumentedListerWatcher(
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces.AllowList, c.config.Namespaces.DenyList, func(namespace string) cache.ListerWatcher {
+				return &cache.ListWatch{
+					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
+						return mclient.MonitoringV1().PrometheusRules(namespace).List(options)
+					},
+					WatchFunc: mclient.MonitoringV1().PrometheusRules(namespace).Watch,
+				}
+			}),
+		),
 		&monitoringv1.PrometheusRule{}, resyncPeriod, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc},
 	)
 
 	c.cmapInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
-			return &cache.ListWatch{
-				ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
-					options.LabelSelector = labelPrometheusName
-					return c.kclient.CoreV1().ConfigMaps(namespace).List(options)
-				},
-				WatchFunc: func(options metav1.ListOptions) (watch.Interface, error) {
-					options.LabelSelector = labelPrometheusName
-					return c.kclient.CoreV1().ConfigMaps(namespace).Watch(options)
-				},
-			}
-		}),
+		c.metrics.NewInstrumentedListerWatcher(
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces.PrometheusAllowList, c.config.Namespaces.DenyList, func(namespace string) cache.ListerWatcher {
+				return &cache.ListWatch{
+					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
+						options.LabelSelector = labelPrometheusName
+						return c.kclient.CoreV1().ConfigMaps(namespace).List(options)
+					},
+					WatchFunc: func(options metav1.ListOptions) (watch.Interface, error) {
+						options.LabelSelector = labelPrometheusName
+						return c.kclient.CoreV1().ConfigMaps(namespace).Watch(options)
+					},
+				}
+			}),
+		),
 		&v1.ConfigMap{}, resyncPeriod, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc},
 	)
 
 	c.secrInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
-			return cache.NewListWatchFromClient(c.kclient.CoreV1().RESTClient(), "secrets", namespace, fields.Everything())
-		}),
+		c.metrics.NewInstrumentedListerWatcher(
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces.PrometheusAllowList, c.config.Namespaces.DenyList, func(namespace string) cache.ListerWatcher {
+				return cache.NewListWatchFromClient(c.kclient.CoreV1().RESTClient(), "secrets", namespace, fields.Everything())
+			}),
+		),
 		&v1.Secret{}, resyncPeriod, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc},
 	)
 
 	c.ssetInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
-			return cache.NewListWatchFromClient(c.kclient.AppsV1().RESTClient(), "statefulsets", namespace, fields.Everything())
-		}),
+		c.metrics.NewInstrumentedListerWatcher(
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces.PrometheusAllowList, c.config.Namespaces.DenyList, func(namespace string) cache.ListerWatcher {
+				return cache.NewListWatchFromClient(c.kclient.AppsV1().RESTClient(), "statefulsets", namespace, fields.Everything())
+			}),
+		),
 		&appsv1.StatefulSet{}, resyncPeriod, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc},
 	)
 
@@ -311,36 +354,19 @@ func New(conf Config, logger log.Logger) (*Operator, error) {
 	// If the only namespace is v1.NamespaceAll, then the client must be
 	// privileged and a regular cache.ListWatch will be used. In this case
 	// watching works and we do not need to resync so frequently.
-	if listwatch.IsAllNamespaces(c.config.Namespaces) {
+	if listwatch.IsAllNamespaces(c.config.Namespaces.AllowList) {
 		nsResyncPeriod = resyncPeriod
 	}
 	c.nsInf = cache.NewSharedIndexInformer(
-		listwatch.NewUnprivilegedNamespaceListWatchFromClient(c.logger, c.kclient.CoreV1().RESTClient(), c.config.Namespaces, c.config.DeniedNamespaces, fields.Everything()),
+		c.metrics.NewInstrumentedListerWatcher(
+			listwatch.NewUnprivilegedNamespaceListWatchFromClient(c.logger, c.kclient.CoreV1().RESTClient(), c.config.Namespaces.AllowList, c.config.Namespaces.DenyList, fields.Everything()),
+		),
 		&v1.Namespace{}, nsResyncPeriod, cache.Indexers{},
 	)
 
 	return c, nil
 }
 
-// RegisterMetrics registers Prometheus metrics on the given Prometheus
-// registerer.
-func (c *Operator) RegisterMetrics(r prometheus.Registerer, reconcileErrorsCounter *prometheus.CounterVec, triggerByCounter *prometheus.CounterVec) {
-	c.reconcileErrorsCounter = reconcileErrorsCounter
-	c.triggerByCounter = triggerByCounter
-
-	c.reconcileErrorsCounter.With(prometheus.Labels{}).Add(0)
-
-	c.nodeAddressLookupErrors = prometheus.NewCounter(prometheus.CounterOpts{
-		Name: "prometheus_operator_node_address_lookup_errors_total",
-		Help: "Number of times a node IP address could not be determined",
-	})
-
-	r.MustRegister(
-		c.nodeAddressLookupErrors,
-		NewPrometheusCollector(c.promInf.GetStore()),
-	)
-}
-
 // waitForCacheSync waits for the informers' caches to be synced.
 func (c *Operator) waitForCacheSync(stopc <-chan struct{}) error {
 	ok := true
@@ -482,7 +508,7 @@ func (c *Operator) handlePrometheusAdd(obj interface{}) {
 	}
 
 	level.Debug(c.logger).Log("msg", "Prometheus added", "key", key)
-	c.triggerByCounter.WithLabelValues(monitoringv1.PrometheusesKind, "add").Inc()
+	c.metrics.TriggerByCounter(monitoringv1.PrometheusesKind, "add").Inc()
 	c.enqueue(key)
 }
 
@@ -493,7 +519,7 @@ func (c *Operator) handlePrometheusDelete(obj interface{}) {
 	}
 
 	level.Debug(c.logger).Log("msg", "Prometheus deleted", "key", key)
-	c.triggerByCounter.WithLabelValues(monitoringv1.PrometheusesKind, "delete").Inc()
+	c.metrics.TriggerByCounter(monitoringv1.PrometheusesKind, "delete").Inc()
 	c.enqueue(key)
 }
 
@@ -508,7 +534,7 @@ func (c *Operator) handlePrometheusUpdate(old, cur interface{}) {
 	}
 
 	level.Debug(c.logger).Log("msg", "Prometheus updated", "key", key)
-	c.triggerByCounter.WithLabelValues(monitoringv1.PrometheusesKind, "update").Inc()
+	c.metrics.TriggerByCounter(monitoringv1.PrometheusesKind, "update").Inc()
 	c.enqueue(key)
 }
 
@@ -571,8 +597,10 @@ func getNodeAddresses(nodes *v1.NodeList) ([]v1.EndpointAddress, []error) {
 }
 
 func (c *Operator) syncNodeEndpointsWithLogError() {
+	c.nodeEndpointSyncs.Inc()
 	err := c.syncNodeEndpoints()
 	if err != nil {
+		c.nodeEndpointSyncErrors.Inc()
 		level.Error(c.logger).Log("msg", "syncing nodes into Endpoints object failed", "err", err)
 	}
 }
@@ -656,7 +684,7 @@ func (c *Operator) handleSmonAdd(obj interface{}) {
 	o, ok := c.getObject(obj)
 	if ok {
 		level.Debug(c.logger).Log("msg", "ServiceMonitor added")
-		c.triggerByCounter.WithLabelValues(monitoringv1.ServiceMonitorsKind, "add").Inc()
+		c.metrics.TriggerByCounter(monitoringv1.ServiceMonitorsKind, "add").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -671,7 +699,7 @@ func (c *Operator) handleSmonUpdate(old, cur interface{}) {
 	o, ok := c.getObject(cur)
 	if ok {
 		level.Debug(c.logger).Log("msg", "ServiceMonitor updated")
-		c.triggerByCounter.WithLabelValues(monitoringv1.ServiceMonitorsKind, "update").Inc()
+		c.metrics.TriggerByCounter(monitoringv1.ServiceMonitorsKind, "update").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -682,7 +710,7 @@ func (c *Operator) handleSmonDelete(obj interface{}) {
 	o, ok := c.getObject(obj)
 	if ok {
 		level.Debug(c.logger).Log("msg", "ServiceMonitor delete")
-		c.triggerByCounter.WithLabelValues(monitoringv1.ServiceMonitorsKind, "delete").Inc()
+		c.metrics.TriggerByCounter(monitoringv1.ServiceMonitorsKind, "delete").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -693,7 +721,7 @@ func (c *Operator) handlePmonAdd(obj interface{}) {
 	o, ok := c.getObject(obj)
 	if ok {
 		level.Debug(c.logger).Log("msg", "PodMonitor added")
-		c.triggerByCounter.WithLabelValues(monitoringv1.PodMonitorsKind, "add").Inc()
+		c.metrics.TriggerByCounter(monitoringv1.PodMonitorsKind, "add").Inc()
 		c.enqueueForNamespace(o.GetNamespace())
 	}
 }
@@ -707,7 +735,7 @@ func (c *Operator) handlePmonUpdate(old, cur interface{}) {
 	o, ok := c.getObject(cur)
 	if ok {
 		level.Debug(c.logger).Log("msg", "PodMonitor updated")
-		c.triggerByCounter.WithLabelValues(monitoringv1.PodMonitorsKind, "update").Inc()
+		c.metrics.TriggerByCounter(monitoringv1.PodMonitorsKind, "update").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -718,7 +746,7 @@ func (c *Operator) handlePmonDelete(obj interface{}) {
 	o, ok := c.getObject(obj)
 	if ok {
 		level.Debug(c.logger).Log("msg", "PodMonitor delete")
-		c.triggerByCounter.WithLabelValues(monitoringv1.PodMonitorsKind, "delete").Inc()
+		c.metrics.TriggerByCounter(monitoringv1.PodMonitorsKind, "delete").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -729,7 +757,7 @@ func (c *Operator) handleRuleAdd(obj interface{}) {
 	o, ok := c.getObject(obj)
 	if ok {
 		level.Debug(c.logger).Log("msg", "PrometheusRule added")
-		c.triggerByCounter.WithLabelValues(monitoringv1.PrometheusRuleKind, "add").Inc()
+		c.metrics.TriggerByCounter(monitoringv1.PrometheusRuleKind, "add").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -744,7 +772,7 @@ func (c *Operator) handleRuleUpdate(old, cur interface{}) {
 	o, ok := c.getObject(cur)
 	if ok {
 		level.Debug(c.logger).Log("msg", "PrometheusRule updated")
-		c.triggerByCounter.WithLabelValues(monitoringv1.PrometheusRuleKind, "update").Inc()
+		c.metrics.TriggerByCounter(monitoringv1.PrometheusRuleKind, "update").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -755,7 +783,7 @@ func (c *Operator) handleRuleDelete(obj interface{}) {
 	o, ok := c.getObject(obj)
 	if ok {
 		level.Debug(c.logger).Log("msg", "PrometheusRule deleted")
-		c.triggerByCounter.WithLabelValues(monitoringv1.PrometheusRuleKind, "delete").Inc()
+		c.metrics.TriggerByCounter(monitoringv1.PrometheusRuleKind, "delete").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -766,7 +794,7 @@ func (c *Operator) handleSecretDelete(obj interface{}) {
 	o, ok := c.getObject(obj)
 	if ok {
 		level.Debug(c.logger).Log("msg", "Secret deleted")
-		c.triggerByCounter.WithLabelValues("Secret", "delete").Inc()
+		c.metrics.TriggerByCounter("Secret", "delete").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -780,7 +808,7 @@ func (c *Operator) handleSecretUpdate(old, cur interface{}) {
 	o, ok := c.getObject(cur)
 	if ok {
 		level.Debug(c.logger).Log("msg", "Secret updated")
-		c.triggerByCounter.WithLabelValues("Secret", "update").Inc()
+		c.metrics.TriggerByCounter("Secret", "update").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -790,7 +818,7 @@ func (c *Operator) handleSecretAdd(obj interface{}) {
 	o, ok := c.getObject(obj)
 	if ok {
 		level.Debug(c.logger).Log("msg", "Secret added")
-		c.triggerByCounter.WithLabelValues("Secret", "add").Inc()
+		c.metrics.TriggerByCounter("Secret", "add").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -801,7 +829,7 @@ func (c *Operator) handleConfigMapAdd(obj interface{}) {
 	o, ok := c.getObject(obj)
 	if ok {
 		level.Debug(c.logger).Log("msg", "ConfigMap added")
-		c.triggerByCounter.WithLabelValues("ConfigMap", "add").Inc()
+		c.metrics.TriggerByCounter("ConfigMap", "add").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -811,7 +839,7 @@ func (c *Operator) handleConfigMapDelete(obj interface{}) {
 	o, ok := c.getObject(obj)
 	if ok {
 		level.Debug(c.logger).Log("msg", "ConfigMap deleted")
-		c.triggerByCounter.WithLabelValues("ConfigMap", "delete").Inc()
+		c.metrics.TriggerByCounter("ConfigMap", "delete").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -825,7 +853,7 @@ func (c *Operator) handleConfigMapUpdate(old, cur interface{}) {
 	o, ok := c.getObject(cur)
 	if ok {
 		level.Debug(c.logger).Log("msg", "ConfigMap updated")
-		c.triggerByCounter.WithLabelValues("ConfigMap", "update").Inc()
+		c.metrics.TriggerByCounter("ConfigMap", "update").Inc()
 
 		c.enqueueForNamespace(o.GetNamespace())
 	}
@@ -952,7 +980,7 @@ func (c *Operator) processNextWorkItem() bool {
 		return true
 	}
 
-	c.reconcileErrorsCounter.With(prometheus.Labels{}).Inc()
+	c.metrics.ReconcileErrorsCounter().Inc()
 	utilruntime.HandleError(errors.Wrap(err, fmt.Sprintf("Sync %q failed", key)))
 	c.queue.AddRateLimited(key)
 
@@ -998,7 +1026,7 @@ func prometheusKeyToStatefulSetKey(key string) string {
 func (c *Operator) handleStatefulSetDelete(obj interface{}) {
 	if ps := c.prometheusForStatefulSet(obj); ps != nil {
 		level.Debug(c.logger).Log("msg", "StatefulSet delete")
-		c.triggerByCounter.WithLabelValues("StatefulSet", "delete").Inc()
+		c.metrics.TriggerByCounter("StatefulSet", "delete").Inc()
 
 		c.enqueue(ps)
 	}
@@ -1007,7 +1035,7 @@ func (c *Operator) handleStatefulSetDelete(obj interface{}) {
 func (c *Operator) handleStatefulSetAdd(obj interface{}) {
 	if ps := c.prometheusForStatefulSet(obj); ps != nil {
 		level.Debug(c.logger).Log("msg", "StatefulSet added")
-		c.triggerByCounter.WithLabelValues("StatefulSet", "add").Inc()
+		c.metrics.TriggerByCounter("StatefulSet", "add").Inc()
 
 		c.enqueue(ps)
 	}
@@ -1028,7 +1056,7 @@ func (c *Operator) handleStatefulSetUpdate(oldo, curo interface{}) {
 
 	if ps := c.prometheusForStatefulSet(cur); ps != nil {
 		level.Debug(c.logger).Log("msg", "StatefulSet updated")
-		c.triggerByCounter.WithLabelValues("StatefulSet", "update").Inc()
+		c.metrics.TriggerByCounter("StatefulSet", "update").Inc()
 
 		c.enqueue(ps)
 	}
@@ -1054,7 +1082,7 @@ func (c *Operator) sync(key string) error {
 	}
 
 	level.Info(c.logger).Log("msg", "sync prometheus", "key", key)
-
+	checkPrometheusSpecDeprecation(key, p, c.logger)
 	ruleConfigMapNames, err := c.createOrUpdateRuleConfigMaps(p)
 	if err != nil {
 		return err
@@ -1085,6 +1113,10 @@ func (c *Operator) sync(key string) error {
 		return err
 	}
 
+	if err := c.createOrUpdateTLSAssetSecret(p); err != nil {
+		return errors.Wrap(err, "creating tls asset secret failed")
+	}
+
 	// Create governing service if it doesn't exist.
 	svcClient := c.kclient.CoreV1().Services(p.Namespace)
 	if err := k8sutil.CreateOrUpdateService(svcClient, makeStatefulSetService(p, c.config)); err != nil {
@@ -1098,7 +1130,12 @@ func (c *Operator) sync(key string) error {
 		return errors.Wrap(err, "retrieving statefulset failed")
 	}
 
-	newSSetInputHash, err := createSSetInputHash(*p, c.config, ruleConfigMapNames)
+	spec := appsv1.StatefulSetSpec{}
+	if obj != nil {
+		ss := obj.(*appsv1.StatefulSet)
+		spec = ss.Spec
+	}
+	newSSetInputHash, err := createSSetInputHash(*p, c.config, ruleConfigMapNames, spec)
 	if err != nil {
 		return err
 	}
@@ -1107,6 +1144,7 @@ func (c *Operator) sync(key string) error {
 	if err != nil {
 		return errors.Wrap(err, "making statefulset failed")
 	}
+	sanitizeSTS(sset)
 
 	if !exists {
 		level.Debug(c.logger).Log("msg", "no current Prometheus statefulset found")
@@ -1129,7 +1167,8 @@ func (c *Operator) sync(key string) error {
 	sErr, ok := err.(*apierrors.StatusError)
 
 	if ok && sErr.ErrStatus.Code == 422 && sErr.ErrStatus.Reason == metav1.StatusReasonInvalid {
-		level.Debug(c.logger).Log("msg", "resolving illegal update of Prometheus StatefulSet")
+		c.metrics.StsDeleteCreateCounter().Inc()
+		level.Info(c.logger).Log("msg", "resolving illegal update of Prometheus StatefulSet", "details", sErr.ErrStatus.Details)
 		propagationPolicy := metav1.DeletePropagationForeground
 		if err := ssetClient.Delete(sset.GetName(), &metav1.DeleteOptions{PropagationPolicy: &propagationPolicy}); err != nil {
 			return errors.Wrap(err, "failed to delete StatefulSet to avoid forbidden action")
@@ -1144,18 +1183,53 @@ func (c *Operator) sync(key string) error {
 	return nil
 }
 
-func createSSetInputHash(p monitoringv1.Prometheus, c Config, ruleConfigMapNames []string) (string, error) {
+// sanitizeSTS removes values for APIVersion and Kind from the VolumeClaimTemplates.
+// This prevents update failures due to these fields changing when applied.
+func sanitizeSTS(sts *appsv1.StatefulSet) {
+	for i := range sts.Spec.VolumeClaimTemplates {
+		sts.Spec.VolumeClaimTemplates[i].APIVersion = ""
+		sts.Spec.VolumeClaimTemplates[i].Kind = ""
+	}
+}
+
+//checkPrometheusSpecDeprecation checks for deprecated fields in the prometheus spec and logs a warning if applicable
+func checkPrometheusSpecDeprecation(key string, p *monitoringv1.Prometheus, logger log.Logger) {
+	deprecationWarningf := "prometheus key=%v, field %v is deprecated, '%v' field should be used instead"
+	if p.Spec.BaseImage != "" {
+		level.Warn(logger).Log("msg", fmt.Sprintf(deprecationWarningf, key, "spec.baseImage", "spec.image"))
+	}
+	if p.Spec.Tag != "" {
+		level.Warn(logger).Log("msg", fmt.Sprintf(deprecationWarningf, key, "spec.tag", "spec.image"))
+	}
+	if p.Spec.SHA != "" {
+		level.Warn(logger).Log("msg", fmt.Sprintf(deprecationWarningf, key, "spec.sha", "spec.image"))
+	}
+	if p.Spec.Thanos != nil {
+		if p.Spec.BaseImage != "" {
+			level.Warn(logger).Log("msg", fmt.Sprintf(deprecationWarningf, key, "spec.thanos.baseImage", "spec.thanos.image"))
+		}
+		if p.Spec.Tag != "" {
+			level.Warn(logger).Log("msg", fmt.Sprintf(deprecationWarningf, key, "spec.thanos.tag", "spec.thanos.image"))
+		}
+		if p.Spec.SHA != "" {
+			level.Warn(logger).Log("msg", fmt.Sprintf(deprecationWarningf, key, "spec.thanos.sha", "spec.thanos.image"))
+		}
+	}
+}
+
+func createSSetInputHash(p monitoringv1.Prometheus, c Config, ruleConfigMapNames []string, ss interface{}) (string, error) {
 	hash, err := hashstructure.Hash(struct {
 		P monitoringv1.Prometheus
 		C Config
+		S interface{}
 		R []string `hash:"set"`
-	}{p, c, ruleConfigMapNames},
+	}{p, c, ss, ruleConfigMapNames},
 		nil,
 	)
 	if err != nil {
 		return "", errors.Wrap(
 			err,
-			"failed to calculate combined hash of Prometheus CRD, config and"+
+			"failed to calculate combined hash of Prometheus StatefulSet, Prometheus CRD, config and"+
 				" rule ConfigMap names",
 		)
 	}
@@ -1267,6 +1341,23 @@ func getCredFromSecret(c corev1client.SecretInterface, sel v1.SecretKeySelector,
 	return extractCredKey(s, sel, cred)
 }
 
+func getCredFromConfigMap(c corev1client.ConfigMapInterface, sel v1.ConfigMapKeySelector, cred string, cacheKey string, cache map[string]*v1.ConfigMap) (_ string, err error) {
+	var s *v1.ConfigMap
+	var ok bool
+
+	if s, ok = cache[cacheKey]; !ok {
+		if s, err = c.Get(sel.Name, metav1.GetOptions{}); err != nil {
+			return "", fmt.Errorf("unable to fetch %s configmap %q: %s", cred, sel.Name, err)
+		}
+		cache[cacheKey] = s
+	}
+
+	if a, ok := s.Data[sel.Key]; ok {
+		return string(a), nil
+	}
+	return "", fmt.Errorf("config %s key %q in configmap %q not found", cred, sel.Key, sel.Name)
+}
+
 func loadBasicAuthSecretFromAPI(basicAuth *monitoringv1.BasicAuth, c corev1client.CoreV1Interface, ns string, cache map[string]*v1.Secret) (BasicAuthCredentials, error) {
 	var username string
 	var password string
@@ -1345,6 +1436,7 @@ func (c *Operator) loadBasicAuthSecrets(
 				}
 				secrets[fmt.Sprintf("serviceMonitor/%s/%s/%d", mon.Namespace, mon.Name, i)] = credentials
 			}
+
 		}
 	}
 
@@ -1381,6 +1473,126 @@ func (c *Operator) loadBasicAuthSecrets(
 
 }
 
+func (c *Operator) loadBearerTokensFromSecrets(mons map[string]*monitoringv1.ServiceMonitor) (map[string]BearerToken, error) {
+	tokens := map[string]BearerToken{}
+	nsSecretCache := make(map[string]*v1.Secret)
+
+	for _, mon := range mons {
+		for i, ep := range mon.Spec.Endpoints {
+			if ep.BearerTokenSecret.Name == "" {
+				continue
+			}
+
+			sClient := c.kclient.CoreV1().Secrets(mon.Namespace)
+			token, err := getCredFromSecret(
+				sClient,
+				ep.BearerTokenSecret,
+				"bearertoken",
+				mon.Namespace+"/"+ep.BearerTokenSecret.Name,
+				nsSecretCache,
+			)
+			if err != nil {
+				return nil, fmt.Errorf(
+					"failed to extract endpoint bearertoken for servicemonitor %v from secret %v in namespace %v",
+					mon.Name, ep.BearerTokenSecret.Name, mon.Namespace,
+				)
+			}
+
+			tokens[fmt.Sprintf("serviceMonitor/%s/%s/%d", mon.Namespace, mon.Name, i)] = BearerToken(token)
+		}
+	}
+
+	return tokens, nil
+}
+
+func (c *Operator) loadTLSAssets(mons map[string]*monitoringv1.ServiceMonitor) (map[string]TLSAsset, error) {
+	assets := map[string]TLSAsset{}
+	nsSecretCache := make(map[string]*v1.Secret)
+	nsConfigMapCache := make(map[string]*v1.ConfigMap)
+
+	for _, mon := range mons {
+		for _, ep := range mon.Spec.Endpoints {
+			if ep.TLSConfig == nil {
+				continue
+			}
+
+			prefix := mon.Namespace + "/"
+			secretSelectors := map[string]*v1.SecretKeySelector{}
+			configMapSelectors := map[string]*v1.ConfigMapKeySelector{}
+			if ep.TLSConfig.CA != (monitoringv1.SecretOrConfigMap{}) {
+				switch {
+				case ep.TLSConfig.CA.Secret != nil:
+					secretSelectors[prefix+ep.TLSConfig.CA.Secret.Name+"/"+ep.TLSConfig.CA.Secret.Key] = ep.TLSConfig.CA.Secret
+				case ep.TLSConfig.CA.ConfigMap != nil:
+					configMapSelectors[prefix+ep.TLSConfig.CA.ConfigMap.Name+"/"+ep.TLSConfig.CA.ConfigMap.Key] = ep.TLSConfig.CA.ConfigMap
+				}
+			}
+			if ep.TLSConfig.Cert != (monitoringv1.SecretOrConfigMap{}) {
+				switch {
+				case ep.TLSConfig.Cert.Secret != nil:
+					secretSelectors[prefix+ep.TLSConfig.Cert.Secret.Name+"/"+ep.TLSConfig.Cert.Secret.Key] = ep.TLSConfig.Cert.Secret
+				case ep.TLSConfig.Cert.ConfigMap != nil:
+					configMapSelectors[prefix+ep.TLSConfig.Cert.ConfigMap.Name+"/"+ep.TLSConfig.Cert.ConfigMap.Key] = ep.TLSConfig.Cert.ConfigMap
+				}
+			}
+			if ep.TLSConfig.KeySecret != nil {
+				secretSelectors[prefix+ep.TLSConfig.KeySecret.Name+"/"+ep.TLSConfig.KeySecret.Key] = ep.TLSConfig.KeySecret
+			}
+
+			for key, selector := range secretSelectors {
+				sClient := c.kclient.CoreV1().Secrets(mon.Namespace)
+				asset, err := getCredFromSecret(
+					sClient,
+					*selector,
+					"tls config",
+					key,
+					nsSecretCache,
+				)
+				if err != nil {
+					return nil, fmt.Errorf(
+						"failed to extract endpoint tls asset for servicemonitor %v from secret %v and key %v in namespace %v",
+						mon.Name, selector.Name, selector.Key, mon.Namespace,
+					)
+				}
+
+				assets[fmt.Sprintf(
+					"%v_%v_%v",
+					mon.Namespace,
+					selector.Name,
+					selector.Key,
+				)] = TLSAsset(asset)
+			}
+
+			for key, selector := range configMapSelectors {
+				sClient := c.kclient.CoreV1().ConfigMaps(mon.Namespace)
+				asset, err := getCredFromConfigMap(
+					sClient,
+					*selector,
+					"tls config",
+					key,
+					nsConfigMapCache,
+				)
+				if err != nil {
+					return nil, fmt.Errorf(
+						"failed to extract endpoint tls asset for servicemonitor %v from configmap %v and key %v in namespace %v",
+						mon.Name, selector.Name, selector.Key, mon.Namespace,
+					)
+				}
+
+				assets[fmt.Sprintf(
+					"%v_%v_%v",
+					mon.Namespace,
+					selector.Name,
+					selector.Key,
+				)] = TLSAsset(asset)
+			}
+
+		}
+	}
+
+	return assets, nil
+}
+
 func (c *Operator) createOrUpdateConfigurationSecret(p *monitoringv1.Prometheus, ruleConfigMapNames []string) error {
 	smons, err := c.selectServiceMonitors(p)
 	if err != nil {
@@ -1403,6 +1615,11 @@ func (c *Operator) createOrUpdateConfigurationSecret(p *monitoringv1.Prometheus,
 		return err
 	}
 
+	bearerTokens, err := c.loadBearerTokensFromSecrets(smons)
+	if err != nil {
+		return err
+	}
+
 	additionalScrapeConfigs, err := c.loadAdditionalScrapeConfigsSecret(p.Spec.AdditionalScrapeConfigs, SecretsInPromNS)
 	if err != nil {
 		return errors.Wrap(err, "loading additional scrape configs from Secret failed")
@@ -1422,6 +1639,7 @@ func (c *Operator) createOrUpdateConfigurationSecret(p *monitoringv1.Prometheus,
 		smons,
 		pmons,
 		basicAuthSecrets,
+		bearerTokens,
 		additionalScrapeConfigs,
 		additionalAlertRelabelConfigs,
 		additionalAlertManagerConfigs,
@@ -1469,9 +1687,70 @@ func (c *Operator) createOrUpdateConfigurationSecret(p *monitoringv1.Prometheus,
 	return err
 }
 
+func (c *Operator) createOrUpdateTLSAssetSecret(p *monitoringv1.Prometheus) error {
+	boolTrue := true
+	sClient := c.kclient.CoreV1().Secrets(p.Namespace)
+
+	smons, err := c.selectServiceMonitors(p)
+	if err != nil {
+		return errors.Wrap(err, "selecting ServiceMonitors failed")
+	}
+
+	tlsAssets, err := c.loadTLSAssets(smons)
+	if err != nil {
+		return err
+	}
+
+	tlsAssetsSecret := &v1.Secret{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:   tlsAssetsSecretName(p.Name),
+			Labels: c.config.Labels.Merge(managedByOperatorLabels),
+			OwnerReferences: []metav1.OwnerReference{
+				{
+					APIVersion:         p.APIVersion,
+					BlockOwnerDeletion: &boolTrue,
+					Controller:         &boolTrue,
+					Kind:               p.Kind,
+					Name:               p.Name,
+					UID:                p.UID,
+				},
+			},
+		},
+		Data: map[string][]byte{},
+	}
+
+	for key, asset := range tlsAssets {
+		tlsAssetsSecret.Data[key] = []byte(asset)
+	}
+
+	_, err = sClient.Get(tlsAssetsSecret.Name, metav1.GetOptions{})
+	if err != nil {
+		if !apierrors.IsNotFound(err) {
+			return errors.Wrapf(
+				err,
+				"failed to check whether tls assets secret already exists for Prometheus %v in namespace %v",
+				p.Name,
+				p.Namespace,
+			)
+		}
+		_, err = sClient.Create(tlsAssetsSecret)
+		level.Debug(c.logger).Log("msg", "created tlsAssetsSecret", "secretname", tlsAssetsSecret.Name)
+
+	} else {
+		_, err = sClient.Update(tlsAssetsSecret)
+		level.Debug(c.logger).Log("msg", "updated tlsAssetsSecret", "secretname", tlsAssetsSecret.Name)
+	}
+
+	if err != nil {
+		return errors.Wrapf(err, "failed to create TLS assets secret for Prometheus %v in namespace %v", p.Name, p.Namespace)
+	}
+
+	return nil
+}
+
 func (c *Operator) selectServiceMonitors(p *monitoringv1.Prometheus) (map[string]*monitoringv1.ServiceMonitor, error) {
 	namespaces := []string{}
-	// Selectors might overlap. Deduplicate them along the keyFunc.
+	// Selectors (<namespace>/<name>) might overlap. Deduplicate them along the keyFunc.
 	res := make(map[string]*monitoringv1.ServiceMonitor)
 
 	servMonSelector, err := metav1.LabelSelectorAsSelector(p.Spec.ServiceMonitorSelector)
@@ -1505,6 +1784,25 @@ func (c *Operator) selectServiceMonitors(p *monitoringv1.Prometheus) (map[string
 		})
 	}
 
+	// If denied by Prometheus spec, filter out all service monitors that access
+	// the file system.
+	if p.Spec.ArbitraryFSAccessThroughSMs.Deny {
+		for namespaceAndName, sm := range res {
+			for _, endpoint := range sm.Spec.Endpoints {
+				if err := testForArbitraryFSAccess(endpoint); err != nil {
+					delete(res, namespaceAndName)
+					level.Warn(c.logger).Log(
+						"msg", "skipping servicemonitor",
+						"error", err.Error(),
+						"servicemonitor", namespaceAndName,
+						"namespace", p.Namespace,
+						"prometheus", p.Name,
+					)
+				}
+			}
+		}
+	}
+
 	serviceMonitors := []string{}
 	for k := range res {
 		serviceMonitors = append(serviceMonitors, k)
@@ -1557,6 +1855,27 @@ func (c *Operator) selectPodMonitors(p *monitoringv1.Prometheus) (map[string]*mo
 	return res, nil
 }
 
+func testForArbitraryFSAccess(e monitoringv1.Endpoint) error {
+	if e.BearerTokenFile != "" {
+		return errors.New("it accesses file system via bearer token file which Prometheus specification prohibits")
+	}
+
+	tlsConf := e.TLSConfig
+	if tlsConf == nil {
+		return nil
+	}
+
+	if err := e.TLSConfig.Validate(); err != nil {
+		return err
+	}
+
+	if tlsConf.CAFile != "" || tlsConf.CertFile != "" || tlsConf.KeyFile != "" {
+		return errors.New("it accesses file system via tls config which Prometheus specification prohibits")
+	}
+
+	return nil
+}
+
 // listMatchingNamespaces lists all the namespaces that match the provided
 // selector.
 func (c *Operator) listMatchingNamespaces(selector labels.Selector) ([]string, error) {
@@ -1572,10 +1891,10 @@ func (c *Operator) listMatchingNamespaces(selector labels.Selector) ([]string, e
 
 func (c *Operator) createCRDs() error {
 	crds := []*extensionsobj.CustomResourceDefinition{
-		k8sutil.NewCustomResourceDefinition(c.config.CrdKinds.Prometheus, c.config.CrdGroup, c.config.Labels.LabelsMap, c.config.EnableValidation),
-		k8sutil.NewCustomResourceDefinition(c.config.CrdKinds.ServiceMonitor, c.config.CrdGroup, c.config.Labels.LabelsMap, c.config.EnableValidation),
-		k8sutil.NewCustomResourceDefinition(c.config.CrdKinds.PodMonitor, c.config.CrdGroup, c.config.Labels.LabelsMap, c.config.EnableValidation),
-		k8sutil.NewCustomResourceDefinition(c.config.CrdKinds.PrometheusRule, c.config.CrdGroup, c.config.Labels.LabelsMap, c.config.EnableValidation),
+		k8sutil.NewCustomResourceDefinition(c.config.CrdKinds.Prometheus, monitoring.GroupName, c.config.Labels.LabelsMap, c.config.EnableValidation),
+		k8sutil.NewCustomResourceDefinition(c.config.CrdKinds.ServiceMonitor, monitoring.GroupName, c.config.Labels.LabelsMap, c.config.EnableValidation),
+		k8sutil.NewCustomResourceDefinition(c.config.CrdKinds.PodMonitor, monitoring.GroupName, c.config.Labels.LabelsMap, c.config.EnableValidation),
+		k8sutil.NewCustomResourceDefinition(c.config.CrdKinds.PrometheusRule, monitoring.GroupName, c.config.Labels.LabelsMap, c.config.EnableValidation),
 	}
 
 	crdClient := c.crdclient.ApiextensionsV1beta1().CustomResourceDefinitions()
@@ -1606,7 +1925,7 @@ func (c *Operator) createCRDs() error {
 	}{
 		{
 			monitoringv1.PrometheusesKind,
-			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces.PrometheusAllowList, c.config.Namespaces.DenyList, func(namespace string) cache.ListerWatcher {
 				return &cache.ListWatch{
 					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 						return c.mclient.MonitoringV1().Prometheuses(namespace).List(options)
@@ -1616,7 +1935,7 @@ func (c *Operator) createCRDs() error {
 		},
 		{
 			monitoringv1.ServiceMonitorsKind,
-			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces.AllowList, c.config.Namespaces.DenyList, func(namespace string) cache.ListerWatcher {
 				return &cache.ListWatch{
 					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 						return c.mclient.MonitoringV1().ServiceMonitors(namespace).List(options)
@@ -1626,7 +1945,7 @@ func (c *Operator) createCRDs() error {
 		},
 		{
 			monitoringv1.PodMonitorsKind,
-			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces.AllowList, c.config.Namespaces.DenyList, func(namespace string) cache.ListerWatcher {
 				return &cache.ListWatch{
 					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 						return c.mclient.MonitoringV1().PodMonitors(namespace).List(options)
@@ -1636,7 +1955,7 @@ func (c *Operator) createCRDs() error {
 		},
 		{
 			monitoringv1.PrometheusRuleKind,
-			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces.AllowList, c.config.Namespaces.DenyList, func(namespace string) cache.ListerWatcher {
 				return &cache.ListWatch{
 					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 						return c.mclient.MonitoringV1().PrometheusRules(namespace).List(options)
